/*
 ͻ
   Program         :  TxtComp                                               
   Version         :  1.20A                                                 
   Author          :  Rob van der Woude                                     
   Download source :  http://www.robvanderwoude.com                         
   Date            :  08/31/97                                              
   Last revision   :  06/06/00                                              
   Function        :  Compares 2 files and creates 2 new files containing   
                      only those lines that were found to be either         
                      different or not present at all in either of the      
                      original files.                                       
   Usage           :  TXTCOMP file1 file2 [/I] [/H] [/N]                    
   Parameters      :  /I     ignores case during compare                    
                      /H     places info in header of files to be created   
                      /L:cc  choose different language, cc may be NL or US  
                      /N     places line number references before each line 
   Remarks         :  TxtComp will create 2 new files:                      
                      FILE1-2.RST contains only those lines of file1 that   
                      were either different from or absent in file2;        
                      FILE2-1.RST contains only those lines of file2 that   
                      were either different from or absent in file1.        
 ͼ
*/

/* Load RexxUtil */
call RxFuncAdd "SysLoadFuncs", "RexxUtil", "SysLoadFuncs"
call SysLoadFuncs

/* Default definitions                              */
/* Names of files to contain differences            */
f1min2 = "FILE1-2.RST"
f2min1 = "FILE2-1.RST"
/* Case sensitive compare, unless /I option is used */
case     = 1
/* No header in files */
header   = 0
/* Language US unless otherwise specified by /L:cc  */
language = "US"
call SetLanguage
/* No line numbers unless /N option is specified    */
numline  = 0

/* Command line parsing                             */
parse upper arg cmdline
do i = 1 by 1 until cmdline = ""
	parse value cmdline with param.i cmdline
end
param.0 = i

cfile.0 = 0
do i = 1 to param.0
	param.i = strip( param.i )
	if left( param.i, 1 ) = "/" then do
		option = substr( param.i, 2, 1 )
		select
			when option = "H" then do
				header = 1
			end
			when option = "I" then do
				case = 0
			end
			when option = "L" then do
				chck = substr( param.i, 3, 1 )
				if chck <> ":" then call Syntax
				lng  = substr( param.i, 4, 2 )
				if lng <> "NL" & lng <> "US" then do
					call Syntax
				end
				else do
					language = lng
				end
			end
			when option = "N" then do
				numline = 1
			end
			otherwise do
				call Syntax
			end
		end
	end
	else do
		files       = cfile.0 + 1
		cfile.0     = files
		cfile.files = param.i
		if cfile.0 > 2 then call Syntax
	end
end
if cfile.0 <> 2 then call Syntax

/* Select language specific message */
if language <> "US" then call SetLanguage

/* Check if input files do exist */
if exist( cfile.1 ) <> 1 | exist( cfile.2 ) <> 1 then do
	call SysCls
	say ""
	call beep 220, 250
	say infile_not_found
	say ""
	say any_key
	call SysCurState "OFF"
	call SysGetKey "NOECHO"
	call SysCls
	call Syntax
end

/* Check if output files already exist */
call diffexist

/* Read first file */
say reading_cfile_1
file.1.  = ""
file.1.0 = 1
file.1.1 = linein( cfile.1, 1, 1 )
if strip( file.1.1 ) = "" & lines( cfile.1 ) = 0 then do
	file.1.0 = 0
end
else do
	i = 1
	do until lines( cfile.1 ) = 0
		i = i + 1
		file.1.i = linein( cfile.1, , 1 )
	end
	file.1.0 = i
end
say cfile_1_read

/* Read second file */
say reading_cfile_2
file.2.  = ""
file.2.0 = 1
file.2.1 = linein( cfile.2, 1, 1 )
if strip( file.2.1 ) = "" & lines( cfile.2 ) = 0 then do
	file.2.0 = 0
end
else do
	i = 1
	do until lines( cfile.2 ) = 0
		i = i + 1
		file.2.i = linein( cfile.2, , 1 )
	end
	file.2.0 = i
end
say cfile_2_read

/* Compare files one way */
file1minus2.  = ""
file1minus2.0 = file.1.0
file2minus1.  = ""
file2minus1.0 = file.2.0
say comparing_1_2
say "   0%"||"1B"X||"[1A"
do i = 1 to file.1.0
	add2file = 1
	do j = 1 to file.2.0
		if case = 0 then do
			if translate( file.1.i ) = translate( file.2.j ) then add2file = 0
		end
		else do
			if file.1.i = file.2.j then add2file = 0
		end
	end
	if add2file = 1 then do
		if numline = 1 then num1minus2.i = right( i||";  ", 8 )
		file1minus2.i = file.1.i
	end
	percentage = ( i * 100 ) % file.1.0
	say right( percentage||"%", 5 )||"1B"X||"[1A"
end
say comp_1_2_ended

say comparing_2_1
say "   0%"||"1B"X||"[1A"
do i = 1 to file.2.0
	add2file = 1
	do j = 1 to file.1.0
		if case = 0 then do
			if translate( file.2.i ) = translate( file.1.j ) then add2file = 0
		end
		else do
			if file.2.i = file.1.j then add2file = 0
		end
	end
	if add2file = 1 then do
		if numline = 1 then num2minus1.i = right( i||";  ", 8 )
		file2minus1.i = file.2.i
	end
	percentage = ( i * 100 ) % file.2.0
	say right( percentage||"%", 5 )||"1B"X||"[1A"
end
say comp_2_1_ended
/* ||"0D0A"X */

j = 0
if file1minus2.0 > 0 then do
	say writing_diff_1_2
	say "   0%"||"1B"X||"[1A"
	if header = 1 then do
		call lineout f1min2, header_1_2
	end
	do i = 1 to file1minus2.0
		if file1minus2.i <> "" then do
			if numline = 1 then do
				call lineout f1min2, num1minus2.i||file1minus2.i
			end
			else do
				call lineout f1min2, file1minus2.i
			end
			j = j + 1
		end
		percentage = ( i * 100 ) % file1minus2.0
		say right( percentage||"%", 5 )||"1B"X||"[1A"
	end
	call lineout f1min2
	say diff_1_2_written
/* ||"0D0A"X */
end

j = 0
if file2minus1.0 > 0 then do
	say writing_diff_2_1
	say "   0%"||"1B"X||"[1A"
	if header = 1 then do
		call lineout f2min1, header_2_1
	end
	do i = 1 to file2minus1.0
		if file2minus1.i <> "" then do
			if numline = 1 then do
				call lineout f2min1, num2minus1.i||file2minus1.i
			end
			else do
				call lineout f2min1, file2minus1.i
			end
			j = j + 1
		end
		percentage = ( i * 100 ) % file2minus1.0
		say right( percentage||"%", 5 )||"1B"X||"[1A"
	end
	call lineout f2min1
	say diff_2_1_written
end
say "1B"X||"[K"
exit


exist: procedure
	call SysFileTree arg( 1 ), "found.", "F"
return found.0


Syntax:
	select
		when language = "NL" then call SyntaxNL
		otherwise call SyntaxUS
	end
return


SyntaxNL:
	call SysCls
	say ""
	say " ͻ"
	say "   Programma      :  TxtComp                                                "
	say "   Versie         :  1.20                                                   "
	say "   Auteur         :  Rob van der Woude                                      "
	say "   Rechten        :  Freeware, 'as is', (C) 1997, Rob van der Woude         "
	say "   Datum          :  31/08/97                                               "
	say "   Laatste rev.   :  03/09/97                                               "
	say "   Functie        :  Vergelijkt 2 bestanden en creert 2 nieuwe waarin      "
	say "                     slechts die regels worden gezet die ofwel van elkaar   "
	say "                     verschillen in de oorspronkelijke bestanden ofwel in   "
	say "                     n van de twee ontbreken.                             "
	say "   Syntax         :  TXTCOMP file1 file2 [/I] [/H] [/N]                     "
	say "   Parameters     :  /I     geen onderscheid hoofd- en kleine letters       "
	say "                     /H     plaatst kopregel met bestandsinformatie         "
	say "                     /L:cc  kies andere taal, cc kan NL of US zijn          "
	say "                     /N     zet regelnummer-referenties voor elke regel     "
	say "   Opmerkingen    :  TxtComp zal 2 nieuwe bestanden creren:                "
	say "                     FILE1-2.RST bevat alleen die regels uit file1 die      "
	say "                     ofwel verschillen van file2 of in file2 ontbreken;     "
	say "                     FILE2-1.RST bevat alleen die regels uit file2 die      "
	say "                     ofwel verschillen van file1 of in file1 ontbreken.     "
	say " ͼ"
	say ""
	say " Druk op een willekeurige toets . . ."||"1B"X||"[1A"
	call SysCurState "OFF"
	call SysGetKey "NOECHO"
	exit
return


SyntaxUS:
	call SysCls
	say ""
	say " ͻ"
	say "   Program        :  TxtComp                                                "
	say "   Version        :  1.20                                                   "
	say "   Author         :  Rob van der Woude                                      "
	say "   Legal status   :  Freeware, as is, (C) 1997, Rob van der Woude           "
	say "   Date           :  08/31/97                                               "
	say "   Last revision  :  09/03/97                                               "
	say "   Function       :  Compares 2 files and creates 2 new files containing    "
	say "                     only those lines that were found to be either          "
	say "                     different or not present at all in either of the       "
	say "                     original files.                                        "
	say "   Usage          :  TXTCOMP file1 file2 [/I] [/H] [/N]                     "
	say "   Parameters     :  /I     ignores case during compare                     "
	say "                     /H     places info in header of files to be created    "
	say "                     /L:cc  choose different language, cc may be NL or US   "
	say "                     /N     places line number references before each line  "
	say "   Remarks        :  TxtComp will create 2 new files:                       "
	say "                     FILE1-2.RST contains only those lines of file1         "
	say "                     that are either different from or absent in file2;     "
	say "                     FILE2-1.RST contains only those lines of file2         "
	say "                     that are either different from or absent in file1.     "
	say " ͼ"
	say ""
	say " Press any key . . ."||"1B"X||"[1A"
	call SysCurState "OFF"
	call SysGetKey "NOECHO"
	exit
return


diffexist:
	call SysFileTree f1min2, "exist1.", "F"
	call SysFileTree f2min1, "exist2.", "F"
	if exist1.0 = 1 | exist2.0 = 1 then do
		call SysCls
		say ""
		call beep 220, 250
		say diff_file_exist
		say remove_or_rename
		say ""
		say any_key
		call SysCurState "OFF"
		call SysGetKey "NOECHO"
		exit
	end
return


SetLanguage:
	select
		when language = "NL" then do
			any_key           = " Druk op een willekeurige toets . . ."
			cfile_1_read      = "1B"X||"[1A "||cfile.1||" ingelezen"||"1B"X||"[K"
			cfile_2_read      = "1B"X||"[1A "||cfile.2||" ingelezen"||"1B"X||"[K"
			comparing_1_2     = " Inhoud "||cfile.1||" wordt vergeleken met "||cfile.2||" . . ."
			comparing_2_1     = " Inhoud "||cfile.2||" wordt vergeleken met "||cfile.1||" . . ."
			comp_1_2_ended    = "1B"X||"[1A Inhoud "||cfile.1||" vergeleken met "||cfile.2||"1B"X||"[K"||"0D0A1B"X||"[K"||"1B"X||"[1A"
			comp_2_1_ended    = "1B"X||"[1A Inhoud "||cfile.2||" vergeleken met "||cfile.1||"1B"X||"[K"||"0D0A1B"X||"[K"||"1B"X||"[1A"
			diff_1_2_written  = "1B"X||"[1A Verschillen tussen "||cfile.1||" en "||cfile.2||" weggeschreven"||"1B"X||"[K"
			diff_2_1_written  = "1B"X||"[1A Verschillen tussen "||cfile.2||" en "||cfile.1||" weggeschreven"||"1B"X||"[K"
			diff_file_exist   = " Bestand "||f1min2||" en/of "||f2min1||" bestaat reeds."
			header_1_2        = "; Regels die wel in "||cfile.1||" maar niet in "||cfile.2||" voorkomen."||"0D0A"X
			header_2_1        = "; Regels die wel in "||cfile.2||" maar niet in "||cfile.1||" voorkomen."||"0D0A"X
			if numline = 1 then do
				header_1_2 = header_1_2||";**** Regelnummers in "||cfile.1||"0D0A"X
				header_2_1 = header_2_1||";**** Regelnummers in "||cfile.2||"0D0A"X
			end
			infile_not_found  = " Opgegeven bestand(en) niet gevonden."
			reading_cfile_1   = " "||cfile.1||" wordt ingelezen . . ."
			reading_cfile_2   = " "||cfile.2||" wordt ingelezen . . ."
			remove_or_rename  = " Verwijder of hernoem de bestanden en probeer opnieuw."
			writing_diff_1_2  = " Verschillen tussen "||cfile.1||" en "||cfile.2||" worden weggeschreven . . ."
			writing_diff_2_1  = " Verschillen tussen "||cfile.2||" en "||cfile.1||" worden weggeschreven . . ."
		end
		otherwise do
			any_key           = " Press any key . . ."
			cfile_1_read      = "1B"X||"[1A "||cfile.1||" read"||"1B"X||"[K"
			cfile_2_read      = "1B"X||"[1A "||cfile.2||" read"||"1B"X||"[K"
			comparing_1_2     = " Comparing "||cfile.1||" with "||cfile.2||" . . ."
			comparing_2_1     = " Comparing "||cfile.2||" with "||cfile.1||" . . ."
			comp_1_2_ended    = " Compare of "||cfile.1||" and "||cfile.2||" ended"
			comp_2_1_ended    = " Compare of "||cfile.2||" and "||cfile.1||" ended"
			diff_1_2_written  = "1B"X||"[1A Differences between "||cfile.1||" and "||cfile.2||" written"||"1B"X||"[K"
			diff_2_1_written  = "1B"X||"[1A Differences between "||cfile.2||" and "||cfile.1||" written"||"1B"X||"[K"
			diff_file_exist   = " File "||f1min2||" and/or "||f2min1||" already exists."
			header_1_2        = "; Lines found in "||cfile.1||" but not in "||cfile.2||"0D0A"X
			header_2_1        = "; Lines found in "||cfile.2||" but not in "||cfile.1||"0D0A"X
			if numline = 1 then do
				header_1_2 = header_1_2||";**** Line numbers as in "||cfile.1||"0D0A"X
				header_2_1 = header_2_1||";**** Line numbers as in "||cfile.2||"0D0A"X
			end
			infile_not_found  = " Input file(s) not found."
			reading_cfile_1   = " Reading "||cfile.1||" . . ."
			reading_cfile_2   = " Reading "||cfile.2||" . . ."
			remove_or_rename  = " Remove or rename these files and try again."
			writing_diff_1_2  = " Writing differences between "||cfile.1||" and "||cfile.2||" to file . . ."
			writing_diff_2_1  = " Writing differences between "||cfile.2||" and "||cfile.1||" to file . . ."
		end
	end
return
